/*
   This file includes weak (i.e. optional) functions to perform SoC related
   initialization. They are:
	1). _nds32_init_ilm():
		Executed before C language initialization to make memory
		ready so that program data can be initialized. An example
		is to initialize ILM.
		Since this is called before C initialization, please
		use provided macros to avoid problems.
	2). __soc_init():
		Further SoC intialization. Called after C language
		initialization, so it is a typical C function.
*/

#include "nds32_intrinsic.h"
#include "config.h"
#ifdef CONFIG_AG101P_16MB
#include "ag101_16mb.h"
#else
#include "ag101.h"
#endif

extern unsigned int _ILM_BASE;
extern unsigned int _ILM_SIZE;
extern unsigned int _DLM_BASE;
extern unsigned int _DLM_SIZE;

#if ILM
/* This must be a leave function, no child funcion. */
void _nds32_init_ilm(void) __attribute__((no_prologue));
void _nds32_init_ilm(void) 	//The function is weak (optional)
{
            /* Set ILM base and enable it */
  //__nds32__mtsr((&_ILM_BASE+1), NDS32_SR_ILMB);
                __nds32__mtsr(0x00600001, NDS32_SR_ILMB);
		__nds32__dsb();
		__nds32__nop();
		__nds32__nop();
		__nds32__nop();	
		__nds32__nop();
		__nds32__nop();	


#ifdef REMAP
	/* remap */
	extern char _edata;
	/* relocation, copy ROM code to ILM  */
	NDS32_FMEMCPY(0x00600000, 0x0, &_edata);
	/* overlap ILM to flash address */
	/* Set ILM base and enable it */
	__nds32__mtsr(0x1, NDS32_SR_ILMB);
	        __nds32__isb();
		__nds32__nop();
		__nds32__nop();
		__nds32__nop();	
		__nds32__nop();
		__nds32__nop();	
		
#endif
}
#endif

void __soc_init()		//The function is weak (optional)
{
  //#if ILM
	uart_init();
	//#endif
}
